----------------------------------------------------------------------------
--  top.vhd
--	Simple VHDL example
--	Version 1.0
--
--  Copyright (C) 2015 H.Poetzl
--
--	This program is free software: you can redistribute it and/or
--	modify it under the terms of the GNU General Public License
--	as published by the Free Software Foundation, either version
--	2 of the License, or (at your option) any later version.
--
--  Vivado 2015.2:
--    mkdir -p build.vivado
--    (cd build.vivado && vivado -mode tcl -source ../vivado.tcl)
----------------------------------------------------------------------------



library IEEE;
use IEEE.std_logic_1164.ALL;
use IEEE.numeric_std.ALL;

library unisim;
use unisim.VCOMPONENTS.ALL;

library unimacro;
use unimacro.VCOMPONENTS.ALL;


entity top is
    port (
	clk_100 : in std_logic;			-- input clock to FPGA
	--
  	ser_rx : in std_logic;			-- serial input
  	ser_tx : out std_logic;			-- serial output
	--
	fifo_reset : in std_logic		-- external fifo reset
    );

end entity top;


architecture RTL of top is

    --------------------------------------------------------------------
    -- PLL Signals
    --------------------------------------------------------------------

    signal pll_fb : std_logic;
    signal pll_clk0 : std_logic;
    signal pll_clk1 : std_logic;

    signal ser_clk : std_logic;
    signal par_clk : std_logic;

    --------------------------------------------------------------------
    -- FIFO Signals
    --------------------------------------------------------------------

    signal ser_rx_data : std_logic_vector (7 downto 0);
    signal ser_rx_full : std_logic;
    signal ser_rx_wen : std_logic;

    signal ser_tx_data : std_logic_vector (7 downto 0);
    signal ser_tx_empty : std_logic;
    signal ser_tx_ren : std_logic;

    signal par_rx_data : std_logic_vector (7 downto 0);
    signal par_rx_empty : std_logic;
    signal par_rx_ren : std_logic;

    signal par_tx_data : std_logic_vector (7 downto 0);
    signal par_tx_full : std_logic;
    signal par_tx_wen : std_logic;

    signal fifo0_rdcount : std_logic_vector (10 downto 0);
    signal fifo0_wrcount : std_logic_vector (10 downto 0);

    signal fifo1_rdcount : std_logic_vector (10 downto 0);
    signal fifo1_wrcount : std_logic_vector (10 downto 0);

begin

    --------------------------------------------------------------------
    -- PLL and Clock Drivers
    --------------------------------------------------------------------

    PLL_inst : PLLE2_BASE
        generic map (
            BANDWIDTH => "LOW",
            CLKFBOUT_MULT => 12,
            CLKIN1_PERIOD => 10.0,
            CLKOUT0_DIVIDE => 120,
            CLKOUT1_DIVIDE => 100,
            DIVCLK_DIVIDE => 1 )
        port map (
            CLKIN1 => clk_100,		-- 100MHz
            CLKFBOUT => pll_fb,
            CLKFBIN => pll_fb,
            CLKOUT0 => pll_clk0,	-- 10MHz
            CLKOUT1 => pll_clk1,	-- 12MHz
            PWRDWN => '0',
            RST => '0' );

    BUFG_inst0 : BUFG
        port map (
            I => pll_clk0,
            O => ser_clk );

    BUFG_inst1 : BUFG
        port map (
            I => pll_clk1,
            O => par_clk );

    --------------------------------------------------------------------
    -- SerDes RX/TX
    --------------------------------------------------------------------

    ser_rxtx_inst : entity work.ser_rxtx
        port map (
            clk => ser_clk,
            --
	    rx => ser_rx,
	    tx => ser_tx,
	    --
	    rx_data => ser_rx_data,
	    rx_full => ser_rx_full,
	    rx_wen => ser_rx_wen,
	    --
	    tx_data => ser_tx_data,
	    tx_empty => ser_tx_empty,
	    tx_ren => ser_tx_ren );

    --------------------------------------------------------------------
    -- Parallel LOOP
    --------------------------------------------------------------------

    par_loop_inst : entity work.par_loop
        port map (
            clk => par_clk,
            --
	    rx_data => par_rx_data,
	    rx_empty => par_rx_empty,
	    rx_ren => par_rx_ren,
	    --
	    tx_data => par_tx_data,
	    tx_full => par_tx_full,
	    tx_wen => par_tx_wen );

    --------------------------------------------------------------------
    -- FIFOs
    --------------------------------------------------------------------

    FIFO_inst0 : FIFO_DUALCLOCK_MACRO
        generic map (
            DEVICE => "7SERIES",
            DATA_WIDTH => 8,
            FIFO_SIZE => "18Kb",
            FIRST_WORD_FALL_THROUGH => TRUE )
        port map (
            DI => ser_rx_data,
            WRCLK => ser_clk,
            WREN => ser_rx_wen,
            WRCOUNT => fifo0_wrcount,
            --
            DO => par_rx_data,
            RDCLK => par_clk,
            RDEN => par_rx_ren,
            RDCOUNT => fifo0_rdcount,
            --
            EMPTY => par_rx_empty,
            FULL => ser_rx_full,
            RST => fifo_reset );


    FIFO_inst1 : FIFO_DUALCLOCK_MACRO
        generic map (
            DEVICE => "7SERIES",
            DATA_WIDTH => 8,
            FIFO_SIZE => "18Kb",
            FIRST_WORD_FALL_THROUGH => TRUE )
        port map (
            DI => par_tx_data,
            WRCLK => par_clk,
            WREN => par_tx_wen,
            WRCOUNT => fifo1_wrcount,
            --
            DO => ser_tx_data,
            RDCLK => ser_clk,
            RDEN => ser_tx_ren,
            RDCOUNT => fifo1_rdcount,
            --
            EMPTY => ser_tx_empty,
            FULL => par_tx_full,
            RST => fifo_reset );

end RTL;
